
in vec3 position_eye, normal_eye;
in vec2 texture_coordinates;
in vec3 fragPos;
uniform float jitter;
uniform vec3 light_pos[10];
uniform mat4 view;
uniform sampler2D basic_texture;
vec3 Ls = vec3 (1.0, 1.0, 0.0); // white specular colour
vec3 Ld = vec3 (0.8, 0.0, 0.8); // diffuse light colour
vec3 La = vec3 (0.2, 0.2, 0.2); // grey ambient colour
vec3 Ks = vec3 (1.0, 1.0, 1.0); // fully reflect specular light
vec3 Kd = vec3 (1.0, 1.0, 1.0); // diffuse surface reflectance
vec3 Ka = vec3 (0.0, 0.0, 0.0); // fully reflect ambient light
float specular_exponent = 10.0; // specular 'power'
out vec4 frag_color;

float find_closest(int x, int y, float c0)
{
	vec4 dither[4];

	dither[0] = vec4( 1.0, 33.0, 9.0, 41.0);
	dither[1] = vec4(49.0, 17.0, 57.0, 25.0);
	dither[2] = vec4(13.0, 45.0, 5.0, 37.0);
	dither[3] = vec4(61.0, 29.0, 53.0, 21.0);

	float limit = 0.0;
	if(x < 4) {
		limit = (dither[x][y]+1.0)/64.0;
	}
	
	if(c0 < limit) {
		return 0.0;
	}else{
		return 1.0;
	}
}

void main () {
	vec3 Ia = La * Ka;
	vec4 result = vec4(0, 0, 0, 1.0);
	for(int i = 0; i < 10; i++) {
		// get dist
		vec3 vPosToLight = fragPos - light_pos[i];
		float dist = length(vPosToLight);
		vPosToLight = normalize(vPosToLight);
		// diffuse light
		vec3 light_position_eye = vec3 (view * vec4 (light_pos[i], 1.0));
		vec3 distance_to_light_eye = light_position_eye - position_eye;
		vec3 direction_to_light_eye = normalize (distance_to_light_eye);
		float dot_prod = dot (direction_to_light_eye, normal_eye);
		dot_prod = max (dot_prod, 0.0);
		vec3 Id = Ld * Kd * dot_prod;
		// attenuation
		float radius = 50.0 + jitter;
		float attC = 1.0;
		float attL = 2.0 / radius;
		float attQ = 1.0 / (radius * radius);
		float attTotal = attC + attL * dist + attQ * dist * dist;
		// specular
		vec3 reflection_eye = reflect (-direction_to_light_eye, normal_eye);
		vec3 surface_to_viewer_eye = normalize (-position_eye);
		float dot_prod_specular = dot (reflection_eye, surface_to_viewer_eye);
		dot_prod_specular = max (dot_prod_specular, 0.0);
		float specular_factor = pow (dot_prod_specular, specular_exponent);
		vec3 Is = Ls * Ks * specular_factor;
		//dithering
		float scale = 0;
	
		vec4 distMod = vec4(1,1,1,1);
		float incr = radius / 7.5;
		if(dist > radius) {
			distMod = vec4(0,0,0,1);
		} else if(dist > radius - incr * 1) {
			distMod = (vec4(Ld,1)/2.2);
			scale = 0.6;
		} else if(dist > radius - incr * 2) {
			distMod = (vec4(Ld,1)/1.8);
			scale = 0.8;
		} else if(dist > radius - incr * 3) {
			distMod =(vec4(Ld,1)/1.4);
			scale = 1;
		} else if(dist > radius - incr * 4) {
			distMod = (vec4(Ld,1)/1.2);
			scale = 2;
		} else {
			distMod = vec4(Ld,1);
		}
		
		vec3 rgb = texture2D(basic_texture, texture_coordinates.xy).rgb;
		
		vec2 xy = gl_FragCoord.xy * scale;
		int x = int(mod(xy.x, 4.0));
		int y = int(mod(xy.y, 4.0));
		
		if( light_pos[i] != vec3(0,0,0) ) {
			vec3 finalRGB;

			finalRGB.r = find_closest(x, y, rgb.r);
			finalRGB.g = find_closest(x, y, rgb.g);
			finalRGB.b = find_closest(x, y, rgb.b);
			
			result += vec4(finalRGB, 0) * distMod;
		}
		
	}
	vec4 texel = texture2D (basic_texture, texture_coordinates);
	frag_color = result * texel;

}